@@ -95,6 +95,17 @@ package Net::BitTorrent;
         return $torrent;
     };
 
+    #
+    has 'ip_filter' => (is       => 'ro',
+                        isa      => 'Net::BitTorrent::Network::IPFilter',
+                        init_arg => undef
+    );
+
+    sub _build_ip_filter {
+        require Net::BitTorrent::Network::IPFilter;
+        Net::BitTorrent::Network::IPFilter->new();
+    }
+
     # DHT (requires udp attribute)
     has 'dht' => (is         => 'ro',
                   isa        => 'Net::BitTorrent::DHT',
@@ -109,7 +120,7 @@ package Net::BitTorrent;
     # Sockets
     has 'port' => (
         is      => 'ro',
-        isa     => 'Int',
+        isa     => 'Int|ArrayRef[Int]',
         default => 0,
         writer  => '_set_port',
 
@@ -122,12 +133,12 @@ package Net::BitTorrent;
                 has $prot
                     . $ipv => (is         => 'ro',
                                init_arg   => undef,
-                               isa        => 'Object',
+                               isa        => 'Maybe[Object]',
                                lazy_build => 1,
                                writer     => '_set_' . $prot . $ipv,
                                predicate  => '_has_' . $prot . $ipv
                     );
-                has $prot
+                has $prot 
                     . $ipv
                     . '_sock' => (
                                  is         => 'ro',
@@ -138,7 +149,7 @@ package Net::BitTorrent;
                                  writer => '_set_' . $prot . $ipv . '_sock',
                                  predicate => '_has_' . $prot . $ipv . '_sock'
                     );
-                has $prot
+                has $prot 
                     . $ipv
                     . '_host' => (
                                  is      => 'ro',
@@ -153,81 +164,149 @@ package Net::BitTorrent;
     after 'BUILD' => sub { $_[0]->$_() for qw[udp6 tcp6 udp4 tcp4] };
 
     sub _build_tcp6 {
-        my ($self) = @_;
-        require Net::BitTorrent::Network::Utility;
-        return Net::BitTorrent::Network::Utility::server(
-            $self->tcp6_host,
-            $self->port,
-            sub { $self->_on_tcp6_in(@_); },
-            sub {
-                my ($sock, $host, $port) = @_;
+        my $s = shift;
+        my $server;
+        for my $port (ref $s->port ? @{$s->port} : $s->port) {
+            require Net::BitTorrent::Network::Utility;
+            $server = Net::BitTorrent::Network::Utility::server(
+                $s->tcp6_host,
+                $port,
+                sub { $s->_on_tcp6_in(@_); },
+                sub {
+                    my ($actual_socket, $actual_host, $actual_port) = @_;
 
-                #if ($self->port != $port) { ...; }
-                $self->_set_tcp6_sock($sock);
-                $self->_set_tcp6_host($host);
-                $self->_set_port($port);
-            },
-            'tcp'
+                    #if ($self->port != $port) { ...; }
+                    $s->_set_tcp6_sock($actual_socket);
+                    $s->_set_tcp6_host($actual_host);
+                    $s->_set_port($actual_port);
+                },
+                'tcp'
+            );
+            last if defined $server;
+        }
+        return $server if $server;
+        $s->trigger_listen_failed(
+            {protocol => 'tcp6',
+             severity => 'fatal',
+             event    => 'listen_failed',
+             message => 'Failed to open IPv6 port to the outside world: ' . $!
+            }
         );
+        return;
     }
 
     sub _build_tcp4 {
-        my ($self) = @_;
-        require Net::BitTorrent::Network::Utility;
-        return Net::BitTorrent::Network::Utility::server(
-            $self->tcp4_host,
-            $self->port,
-            sub { $self->_on_tcp4_in(@_); },
-            sub {
-                my ($sock, $host, $port) = @_;
-                if ($self->port != $port) { ...; }
-                $self->_set_tcp4_sock($sock);
-                $self->_set_tcp4_host($host);
-                $self->_set_port($port);
-            },
-            'tcp'
+        my $s = shift;
+        my $server;
+        for my $port (ref $s->port ? @{$s->port} : $s->port) {
+            require Net::BitTorrent::Network::Utility;
+            $server = Net::BitTorrent::Network::Utility::server(
+                $s->tcp4_host,
+                $port,
+                sub { $s->_on_tcp4_in(@_); },
+                sub {
+                    my ($actual_socket, $actual_host, $actual_port) = @_;
+
+                    #if ($self->port != $port) { ...; }
+                    $s->_set_tcp4_sock($actual_socket);
+                    $s->_set_tcp4_host($actual_host);
+                    $s->_set_port($actual_port);
+                },
+                'tcp'
+            );
+            last if defined $server;
+        }
+        return $server if $server;
+        $s->trigger_listen_failed(
+            {protocol => 'tcp4',
+             severity => 'fatal',
+             event    => 'listen_failed',
+             message => 'Failed to open IPv4 port to the outside world: ' . $!
+            }
         );
+        return;
     }
 
     sub _build_udp6 {
-        my ($self) = @_;
-        require Net::BitTorrent::Network::Utility;
-        return Net::BitTorrent::Network::Utility::server(
-            $self->udp6_host,
-            $self->port,
-            sub { $self->_on_udp6_in(@_); },
-            sub {
-                my ($sock, $host, $port) = @_;
+        my $s = shift;
+        my $server;
+        for my $port (ref $s->port ? @{$s->port} : $s->port) {
+            require Net::BitTorrent::Network::Utility;
+            $server = Net::BitTorrent::Network::Utility::server(
+                $s->udp6_host,
+                $port,
+                sub { $s->_on_udp6_in(@_); },
+                sub {
+                    my ($actual_socket, $actual_host, $actual_port) = @_;
 
-                #if ($self->port != $port) { ...; }
-                $self->_set_udp6_sock($sock);
-                $self->_set_udp6_host($host);
-                $self->_set_port($port);
-            },
-            'udp'
+                    #if ($self->port != $port) { ...; }
+                    $s->_set_udp6_sock($actual_socket);
+                    $s->_set_udp6_host($actual_host);
+                    $s->_set_port($actual_port);
+                },
+                'udp'
+            );
+            last if defined $server;
+        }
+        return $server if $server;
+        $s->trigger_listen_failed(
+            {protocol => 'udp6',
+             severity => 'fatal',
+             event    => 'listen_failed',
+             message => 'Failed to open IPv6 port to the outside world: ' . $!
+            }
         );
+        return;
     }
 
     sub _build_udp4 {
-        my ($self) = @_;
-        require Net::BitTorrent::Network::Utility;
-        return Net::BitTorrent::Network::Utility::server(
-            $self->udp4_host,
-            $self->port,
-            sub { $self->_on_udp4_in(@_); },
-            sub {
-                my ($sock, $host, $port) = @_;
-                if ($self->port != $port) { ...; }
-                $self->_set_udp4_sock($sock);
-                $self->_set_udp4_host($host);
-                $self->_set_port($port);
-            },
-            'udp'
+        my $s = shift;
+        my $server;
+        for my $port (ref $s->port ? @{$s->port} : $s->port) {
+            require Net::BitTorrent::Network::Utility;
+            $server = Net::BitTorrent::Network::Utility::server(
+                $s->udp4_host,
+                $port,
+                sub { $s->_on_udp4_in(@_); },
+                sub {
+                    my ($actual_socket, $actual_host, $actual_port) = @_;
+
+                    #if ($self->port != $port) { ...; }
+                    $s->_set_udp4_sock($actual_socket);
+                    $s->_set_udp4_host($actual_host);
+                    $s->_set_port($actual_port);
+                },
+                'udp'
+            );
+            last if defined $server;
+        }
+        return $server if $server;
+        $s->trigger_listen_failed(
+            {protocol => 'udp4',
+             severity => 'fatal',
+             event    => 'listen_failed',
+             message => 'Failed to open IPv4 port to the outside world: ' . $!
+            }
         );
+        return;
     }
 
     sub _on_tcp4_in {
         my ($self, $peer, $paddr, $host, $port) = @_;
+        my $range = $self->ip_filter->is_banned($host);
+        if (defined $range) {
+            $self->trigger_ip_filter(
+                     {protocol => 'tcp4',
+                      severity => 'debug',
+                      event    => 'ip_filter',
+                      ip       => $host,
+                      range    => $range,
+                      message => 'Incoming connection was blocked by ipfilter'
+                     }
+            );
+            shutdown $peer, 2;
+            return close $peer;
+        }
         require Net::BitTorrent::Peer;
         $self->add_peer(
                     Net::BitTorrent::Peer->new(fh => $peer, client => $self));
@@ -235,22 +314,61 @@ package Net::BitTorrent;
 
     sub _on_tcp6_in {
         my ($self, $peer, $paddr, $host, $port) = @_;
-
-        #use Data::Dump;
-        #ddx \@_;
-        #...;
+        my $range = $self->ip_filter->is_banned($host);
+        if (defined $range) {
+            $self->trigger_ip_filter(
+                     {protocol => 'tcp4',
+                      severity => 'debug',
+                      event    => 'ip_filter',
+                      ip       => $host,
+                      range    => $range,
+                      message => 'Incoming connection was blocked by ipfilter'
+                     }
+            );
+            shutdown $peer, 2;
+            return close $peer;
+        }
+        require Net::BitTorrent::Peer;
+        $self->add_peer(
+                    Net::BitTorrent::Peer->new(fh => $peer, client => $self));
     }
 
     sub _on_udp4_in {
-        my $self = shift;
+        my $s = shift;
         my ($udp, $sock, $paddr, $host, $port, $data, $flags) = @_;
-        $self->dht->_on_udp4_in(@_);
+        my $range = $s->ip_filter->is_banned($host);
+        if (defined $range) {
+            $s->trigger_ip_filter(
+                           {protocol => 'udp4',
+                            severity => 'debug',
+                            event    => 'ip_filter',
+                            ip       => $host,
+                            range    => $range,
+                            message => 'Incoming data was blocked by ipfilter'
+                           }
+            );
+            return;
+        }
+        $s->dht->_on_udp4_in(@_);
     }
 
     sub _on_upd6_in {
-        my $self = shift;
+        my $s = shift;
         my ($udp, $sock, $paddr, $host, $port, $data, $flags) = @_;
-        $self->dht->_on_udp6_in(@_);
+        my $range = $s->ip_filter->is_banned($host);
+        if (defined $range) {
+            $s->trigger_ip_filter(
+                           {protocol => 'upd6',
+                            severity => 'debug',
+                            event    => 'ip_filter',
+                            ip       => $host,
+                            range    => $range,
+                            message => 'Incoming data was blocked by ipfilter'
+                           }
+            );
+            return;
+        }
+        $s->dht->_on_udp6_in(@_);
     }
 
     #
@@ -423,6 +541,6 @@ L<clarification of the CCA-SA3.0|http://creativecommons.org/licenses/by-sa/3.0/u
 Neither this module nor the L<Author|/Author> is affiliated with BitTorrent,
 Inc.
 
-=for rcs $Id: BitTorrent.pm a9b7d6f 2010-06-29 05:55:21Z sanko@cpan.org $
+=for rcs $Id: BitTorrent.pm 5d05177 2010-07-03 18:25:20Z sanko@cpan.org $
 
 =cut
